﻿<?php
/*
 * IP-Symcon JSON RPC 2.0a
 *
 * JSON-RPC 2.0 Specification
 * http://groups.google.com/group/json-rpc/web/json-rpc-2-0
 */

ini_set('display_errors', 1);
ini_set('error_reporting', E_ALL);

//Generate SMD description if parameter is set
if(isset($_GET['gen']) && ($_GET['gen'] == "smd"))
{

	$smd = Array(
		"transport" => "POST",
		"envelope" => "JSON-RPC-2.0",
		"target" => "json-rpc.php"
	);
	
	$typestr = Array("boolean", "integer", "float", "string", "any", "array");
	
	//generate services
	$functionNames = IPS_GetFunctionList(0);
	asort($functionNames);
	foreach($functionNames as $functionName)
	{
		$function = IPS_GetFunction($functionName);
		$parameters = Array();
		foreach($function['Parameters'] as $parameter)
			$parameters[] = Array( "type" => $typestr[$parameter['Type_']] );
		$returns = Array( "type" => $typestr[$function['Result']['Type_']] );

		$services[$function['FunctionName']] = Array("parameters" => $parameters, "returns" => $returns);
	}
	
	$smd['services'] = $services;
	
	file_put_contents("ips.smd", json_encode($smd));
	echo "OK";
	return;
}	

// Check for existing json_encode/json_decode functions.
// are available.
if ( !function_exists('json_encode') || !function_exists('json_decode') )
{
	die("{\"jsonrpc\":\"2.0\",\"error\":{\"code\":\"-32603\",\"message\":\"JSON RPC not available\",\"data\":\"JSON PHP functions are not available.\"},\"id\":\"Null\"}");
}

$postdata; //= $HTTP_RAW_POST_DATA;
if(empty($postdata)) {
	$postdata = $_SERVER['QUERY_STRING'];
}
$request = json_decode($postdata);

//Set custom error handler to trap possible errors and display via json-rpc
set_error_handler(function($errno, $errstr, $errfile, $errline) use($request)
{
	die(createErrorObj($request->{'id'}, "-32601", "Unknown error", $errstr));	
});

// Check if we got a bad request.
if ( $request == null )
{
	echo createErrorObj("Null", "-32700", "Invalid request", "Invalid header or postdata.");
	exit;
}

// Check if we got an invalid request object.
if (! isset($request->{'jsonrpc'}) ||
    ! isset($request->{'method'}) ||
    ! isset($request->{'params'}) ||
    ! isset($request->{'id'}) )
{
	echo createErrorObj("Null", "-32700", "Invalid request", "Invalid request object.");
	exit;
}

if ( ($request->{'method'} != "isAPIReady") && !IPS_FunctionExists($request->{'method'}) )
{
	echo createErrorObj($request->{'id'}, "-32601", "Method not found", "Only IP-Symcon methods are allowed for JSON-RPC.");
	exit;
}

// Check if the given method is callable.
if ( is_callable($request->{'method'}, false, $functionname) )
{
	//register new error handler to catch all invocation errors to IP-Symcon
	set_error_handler(function($errno, $errstr, $errfile, $errline) use($request)
	{
		die(createErrorObj($request->{'id'}, "-32601", "Method execution error", $errstr));	
	});

	//invoke our method
	$result = call_user_func_array($functionname, $request->{'params'});
	
	//restore the error handler to the custom defined one
	restore_error_handler();
	
	//build result object
	echo createReturnObj("0", $request->{'id'}, $result);
} 
else
{
    die(createErrorObj($request->{'id'}, "-32601", "Method not found", "The method does not exist / is not available."));
}

/*
 * Method to check if the API and/or IP-Symcon server is ready to
 * serve requests.
 */
function isAPIReady()
{
	return true;
}

/****************************************************************************
 * INTERNAL FUNCTIONS WHICH ARE NEEDED FOR JSON-RPC OPERATIONS
 ****************************************************************************/

/*
 * json_encode() can only handle utf8 strings. So we have to convert
 * existing strings.
 *
 * @param &item REFERENCE to the item.
 * @param key   The KEY of the item.
 */
function utf8_conv(&$item, $key)
{
	if ( is_string($item) )
		$item = utf8_encode($item);
}

/*
 * Creates a JSON error object, which can be used within the return object.
 *
 * @param id      The id of the request object.
 * @param code    A Number that indicates the actual error that occurred.
 *                This MUST be an integer.
 * @param message A String providing a short description of the error.
 *                The message SHOULD be limited to a concise single sentence.
 * @param data    Additional information, may be omitted. Its contents is entirely
 *                defined by the application (e.g. detailed error
 *                information, nested errors etc.).
 */
function createErrorObj($id, $code, $message, $data)
{
   $arr = array(
      "code"    => $code,
      "message" => $message,
      "data"    => $data
   ); 

   // If we got a parse error, the id has to be "Null" (JSON-RPC RFC).
   if ( $code == "-32700" )
      $id = "Null";

   return createReturnObj("1", $id, $arr);
} 

/*
 * Creates a JSON return object, which will be returned to the sender.
 *
 * @param mode 0 = successfull return, 1 error return.
 * @param id   ID of the request object.
 * @param obj  JSON object which will be returned.
 */
function createReturnObj($mode, $id, $obj)
{
   // Version of this JSON-RPC object.
   $jsonrpc = "2.0";

   // First we have to check, if it's a error/standard object.
   if ( $mode == "0" )
   {
      $arr = array(
         "jsonrpc" => $jsonrpc,
         "result"  => $obj,
         "id"      => $id
      );
   } else
   {
      $arr = array(
         "jsonrpc" => $jsonrpc,
         "error"   => $obj,
         "id"      => $id
      );
   }

   array_walk_recursive($arr, 'utf8_conv');
   return json_encode($arr);
}

?>